package org.mote.wiimote.whiteboard.gui;

import java.awt.Point;
import java.awt.SystemColor;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JPanel;

import net.miginfocom.swing.MigLayout;

import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.mote.wiimote.whiteboard.WiimoteDataHandler;
import org.mote.wiimote.whiteboard.WiimoteWhiteboard;
import org.mote.wiimote.whiteboard.WiimoteDataHandler.WiimoteDataListener;
import org.mote.wiimote.whiteboard.ds.IRDot;
import org.mote.wiimote.whiteboard.ds.Wiimote;
import org.mote.wiimote.whiteboard.preferences.WWPreferences;
import org.mote.wiimote.whiteboard.util.Util;

@SuppressWarnings("serial")
public class CameraMonitor extends JDialog implements WiimoteDataListener {

	private static final long REPAINT_FREQ = 1000 / 25;

	private JPanel canvas;
	private IRDot[][] lights = new IRDot[WWPreferences.WIIMOTES+1][4];
	private LightLabel[][] labels = new LightLabel[WWPreferences.WIIMOTES+1][4];

	public CameraMonitor(WiimoteDataHandler dh) {
		super(Application.getInstance(WiimoteWhiteboard.class).getMainFrame(), Util.getResourceMap(CameraMonitor.class).getString("monitor.Action.text"));
		getRootPane().putClientProperty("Window.style", "small");
		setLayout(new MigLayout());
		
		dh.addWiimoteDataListener(this);

		canvas = new JPanel(null, true);
		canvas.setOpaque(true);
		canvas.setBorder(BorderFactory.createLineBorder(SystemColor.inactiveCaptionBorder));
		add(canvas, "w 50sp, h 50sp, grow, push");
		
		addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
					setVisible(false);
				}
			}
		});

		pack();
		setLocationRelativeTo(null);

		new Timer(true).schedule(new UpdateTask(), 0, REPAINT_FREQ);
	}

	@Action
	public void monitor() {
		setVisible(true);
	}

	private class UpdateTask extends TimerTask {
		@Override
		public void run() {
			if (isVisible()) {
				for (int i = 1; i <= WWPreferences.WIIMOTES; i++)
					for (int j = 0; j < 4; j++) {
						if (labels[i][j] != null)
							labels[i][j].update();
					}
			}
		}
	}

	public void irLights(Wiimote wiimote, IRDot[] lights) {
		this.lights[wiimote.getId()] = lights;
	}
	
	public void irWarped(Map<Wiimote, IRDot[]> data, Point[] warped) {
	}

	public void batteryLevel(Wiimote wiimote, double level) {
	}

	public void wiimoteConnected(Wiimote wiimote) {
		for (int i = 0; i < 4; i++)
			canvas.add(labels[wiimote.getId()][i] = new LightLabel(canvas, lights, wiimote.getId(), i + 1));
	}

	public void wiimoteDisconnected(Wiimote wiimote) {
		for (int i = 0; i < 4; i++) {
			canvas.remove(labels[wiimote.getId()][i]);
			labels[wiimote.getId()][i] = null;
		}
	}

}
