package org.mote.wiimote.whiteboard.mouse;

import java.awt.Point;

import org.mote.wiimote.whiteboard.mouse.rightclick.RightClickStrategy;
import org.mote.wiimote.whiteboard.preferences.WWPreferences;

public class TouchpadControlStrategy implements CursorControlStrategy {
	
	private static final WWPreferences prefs = WWPreferences.getPreferences();
	private RightClickStrategy rcs = WWPreferences.RIGHT_CLICK_STRATEGY;
	
	private long lightOnTime = 0, lightOffTime = System.currentTimeMillis();
	
	public void process(Point p) {
		
		final long now = System.currentTimeMillis();
		final boolean lightOnNow = p != null;
		final boolean lightOnBefore = lightOnTime > lightOffTime;
		
		rcs.process(p);
		
		if (!lightOnBefore && lightOnNow) {
			lightOnTime = now;
		} else if (lightOnBefore && !lightOnNow) {
			lightOffTime = now;
		}
		
		// release mouse button
		if (!lightOnNow && Mouse.LEFT_BUTTON.isPressed() && now - lightOffTime > WWPreferences.SHORT_DELAY) {
			Mouse.LEFT_BUTTON.setPressed(false);
		}
		
		if (lightOnNow) {
			Mouse.move(p);
			if (!prefs.isLeftClick() || (prefs.isRightClick() && rcs.trigger())) {
				return;
			}
		}

		if (lightOnBefore && !lightOnNow) {
			// short flash => left click
			if (lightOffTime - lightOnTime < WWPreferences.SHORT_DELAY) {
				if (Mouse.LEFT_BUTTON.isPressed()) {
					Mouse.LEFT_BUTTON.setPressed(false);
				}
				Mouse.LEFT_BUTTON.setPressed(true);
			} else {
				Mouse.LEFT_BUTTON.setPressed(false);
			}
			
		}
	}

}
